property_codes = {
    "thermal expansion coefficient x direction": "therm_exp_coeff",
    "density": "density",
    "specific heat capacity": "specific_heat",
    "molar mass": "molecular_weight",
    "thermal conductivity x direction": "thermal_conductivity",
    "viscosity": "viscosity",
    "speed of sound": "speed_of_sound",
}
