property_codes = {
    "young's modulus x direction": "EX",
    "young's modulus y direction": "EY",
    "young's modulus z direction": "EZ",
    "secant thermal expansion coefficient x direction": "ALPX",
    "secant thermal expansion coefficient y direction": "ALPY",
    "secant thermal expansion coefficient z direction": "ALPZ",
    "instantaneous thermal expansion coefficient x direction": "CTEX",
    "instantaneous thermal expansion coefficient y direction": "CTEY",
    "instantaneous thermal expansion coefficient z direction": "CTEZ",
    "strain reference temperature": "REFT",
    "poisson's ratio xy": "PRXY",
    "poisson's ratio yz": "PRYZ",
    "poisson's ratio xz": "PRXZ",
    "shear modulus xy": "GXY",
    "shear modulus yz": "GYZ",
    "shear modulus xz": "GXZ",
    "coefficient of friction": "MU",
    "density": "DENS",
    "specific heat capacity": "C",
    "enthalpy": "ENTH",
    "thermal conductivity x direction": "KXX",
    "thermal conductivity y direction": "KYY",
    "thermal conductivity z direction": "KZZ",
    "convection coefficient": "HF",
    "emissivity": "EMIS",
    "heat generation rate": "QRATE",
    "viscosity": "VISC",
    "speed of sound": "SONC",
    "electrical resistivity x direction": "RSVX",
    "electrical resistivity y direction": "RSVY",
    "electrical resistivity z direction": "RSVZ",
    "electric relative permittivity x direction": "PERX",
    "electric relative permittivity y direction": "PERY",
    "electric relative permittivity z direction": "PERZ",
    "magnetic relative permeability x direction": "MURX",
    "magnetic relative permeability y direction": "MURY",
    "magnetic relative permeability z direction": "MURZ",
    "magnetic coercive force x direction": "MGXX",
    "magnetic coercive force y direction": "MGYY",
    "magnetic coercive force z direction": "MGZZ",
    "dielectric loss tangent": "LSST",
    "seebeck coefficient x direction": "SBKX",
    "seebeck coefficient y direction": "SBKY",
    "seebeck coefficient z direction": "SBKZ",
}
